<?php

namespace Drupal\entities_import;

/**
 * Defines interface DataStorageInterface.
 */
interface DataStorageInterface {

  /**
   * Builds a function to save and update.
   *
   * This method is responsible for saving and updating the content and taxonomy from excel rows.
   *
   * @param array $item
   *   array contains excel data.
   * @param array &$context
   *   array for batch process.  
   *   
   */  
  public static function save($item, &$context);

  /**
   * Save and update the paragraph field.
   *
   * This method is responsible for saving and updating the paragraph field with hierarchy structure.
   *
   * @param array $item
   *   array contains excel data.
   * @param string $p_type
   *   array for batch process.
   * @param object $node
   *   node object.
   * @param string $entity_para_field
   *   name of the field.
   *  
   * @return array
   *   array contains paragarph details. 
   */
  public static function updateParagraphDetails($items, $p_type, $node, $entity_para_field);

  /**
   * To get the child paragarph details.
   *
   * This method is responsible for getting the deatils of child paragarph.
   *
   * @param array $item
   *   array contains excel data.
   * @param string $p_type
   *   array for batch process.
   * @param array $paragraph
   *   deatils of child paragraph.   
   *   
   * @return array
   *   array contains paragarph details.
   */
  public static function getParagraphDetails($items, $p_type, $paragraph);  
  
}
