<?php

namespace Drupal\entities_import;

/**
 * Defines interface EntityDetailsInterface.
 */
interface EntityDetailsInterface {

  /**
   * Builds a function to check node/taxonomy exists or not.
   *
   * This method is responsible for checking node/taxonomy exists by given properties.
   *
   * @param array $properties
   *   array contains term deatils to filter.
   * @param string entity_type
   *   name of the entity type. ex: node, taxonomy_term.   
   * 
   * @return object
   *   node object. 
   */  
  public static function checkNodeExistsByProperties($properties, $entity_type);

  /**
   * Builds a function to get the taxonomy term object.
   *
   * This method is responsible for get the taxonomy term object by given properties.
   *
   * @param array $properties
   *   array contains term deatils to filter.  
   * 
   * @return object
   *   term object. 
   */
  public static function getTermByProperties($properties);

  /**
   *  Builds a function to check entity exists or not.
   *
   * This method is responsible for get entity object by given properties.
   *
   * @param array $properties
   *   array contains entity deatils to filter.  
   * @param string entity_type
   *   name of the entity type. ex: node, taxonomy_term. 
   * 
   * @return object
   *   entity object. 
   */
  public static function checkEntityExistsByProperties($properties, $entity_type);  
   
}
