<?php

namespace Drupal\entities_import;

/**
 * Defines interface FieldDetailsInterface. 
 */
interface FieldDetailsInterface {

  /**
   * To get the reference field's list.
   *
   * This method is responsible to get the list of reference fields.
   *
   * @param string $entity
   *   name of the entity. Ex: node
   * @param string $bundle
   *   name of the bundele. Ex: article.   
   * 
   * @return array
   *   array of field list.
   */  
  public static function getReferenceFieldBundle($entity, $bundle);

  /**
   * To get the cardinality details.
   *
   * This method is responsible to get cardinality details of all the fields.
   *
   * @param string $entity
   *   name of the entity. Ex: node
   * @param string $bundle
   *   name of the bundele. Ex: article.   
   * 
   * @return array
   *   array of field list.
   */  
  public static function getFieldCardinality($entity, $bundle);

  /**
   * To get type of the field.
   *
   * This method is responsible to get type of the given field. Ex: text, entity_reference_revisions.
   *
   * @param string $entity
   *   name of the entity. Ex: node
   * @param string $bundle
   *   name of the bundele. Ex: article.   
   * @param string $field_name
   *   name of the field. Ex: field_image.
   * 
   * @return string
   *   name of the field.
   */  
  public static function getFieldType($entity, $bundle, $field_name);
  
}
