<?php

namespace Drupal\entities_import;

use Drupal\file\Entity\File;

/**
 * Defines interface FileDetailsInterface. 
 */
interface FileDetailsInterface {  

  /**
   * To create a new file.
   *
   * This method is responsible for creating new files for each content/term with multiple files upload.
   *
   * @param string $filename
   *   name of the file in the excel.
   * @param string $file_folder_path
   *   Folder path to store the files inside default files folder. Ex: article-images.   
   * 
   * @return int
   *   ID of the file.
   */  
  public static function create($filename, $file_folder_path); 
  
  /**
   * Function to update the file with excel file fields.
   *
   * This method is responsible for updating the existing files for each content/term with multiple files upload.
   *
   * @param array $items
   *   details of the file.
   * @param array $fileIDs
   *   array of the file IDs.
   * @param string $file_folder_path
   *   Folder path to store the files inside default files folder. Ex: article-images.   
   * 
   * @return array
   *   array of the file IDs.
   */  
  public static function Update($items, $fileIDs, $file_folder_path);  

  /**
   * Delete the unnecessary files.
   *
   * This method is responsible to delete the unnecessary files by checking with existing data while importing.
   *   
   * @param array $items
   *   details of the file.
   *    
   */  
  public static function delete($fids);
  
}
